import React from 'react';
import ReactDOM from 'react-dom';
import TestUtils from 'react-addons-test-utils';
import purebem from 'purebem';
import { mount } from 'enzyme';

import Dropdown from './Dropdown';

import dropdownValueOptionState from './option-value/dropdownValueOptionState';


const block = purebem.of('test-dropdown');
// Used to find options to interact with. Far from ideal, but at least
// the tests will fail if the style is refactored.
const optionElement = block('option');

const TestDropdown = Dropdown({
    block,
    displayName: 'TestDropdown'
});

const getOptions = (a = false, b = false, c = false) => {
    return [
        dropdownValueOptionState({
            disabled: a,
            text: 'A',
            value: 'a'
        }),
        dropdownValueOptionState({
            disabled: b,
            text: 'B',
            value: 'b'
        }),
        dropdownValueOptionState({
            disabled: c,
            text: 'C',
            textSelected: 'C (selected)',
            value: 'c'
        })
    ];
};

describe('Dropdown', () => {

    it('should render empty text if no options', () => {
        const wrapper = mount(
            <TestDropdown
                emptyText="Empty"
                options={ [] } />
        );

        expect(wrapper.text()).toBe('Empty');
    });

    [
        { name: 'should render empty text if value is undefined', data: { value: undefined } },
        { name: 'should render empty text if value is null', data: { value: null } },
        { name: 'should render empty text if value is empty string', data: { value: '' } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const wrapper = mount(
                <TestDropdown
                    emptyText="Empty"
                    options={ getOptions() }
                    value={ fixture.data.value } />
            );

            expect(wrapper.text()).toBe('Empty');
        });
    });

    [
        { name: 'should render empty text if option value is undefined', data: { value: undefined } },
        { name: 'should render empty text if option value is null', data: { value: null } },
        { name: 'should render empty text if option value is empty string', data: { value: '' } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const wrapper = mount(
                <TestDropdown
                    emptyText="Empty"
                    options={ [dropdownValueOptionState({ value: fixture.data.value })] } />
            );

            expect(wrapper.text()).toBe('Empty');
        });
    });

    it('should render option text of dropdown value', () => {
        const wrapper = mount(
            <TestDropdown
                options={ getOptions() }
                value="b" />
        );

        expect(wrapper.text()).toBe('B');
    });

    it('should render option selected text of dropdown value', () => {
        const wrapper = mount(
            <TestDropdown
                options={ getOptions() }
                value="c" />
        );

        expect(wrapper.text()).toBe('C (selected)');
    });

    it('should expand collapsed dropdown when clicked', () => {
        const onExpand = jest.fn();

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ false }
                onExpand={ onExpand }
                options={ getOptions() } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.click(node);

        expect(onExpand).toHaveBeenCalled();
    });

    [
        { name: 'should expand collapsed dropdown when using enter', data: { which: 13 } },
        { name: 'should expand collapsed dropdown when using space', data: { which: 32 } },
        { name: 'should expand collapsed dropdown when using down arrow', data: { which: 40 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onExpand = jest.fn();

            const dropdown = TestUtils.renderIntoDocument(
                <TestDropdown
                    expanded={ false }
                    onExpand={ onExpand }
                    options={ getOptions() } />
            );

            const node = ReactDOM.findDOMNode(dropdown);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onExpand).toHaveBeenCalled();
        });
    });

    it('should collapse expanded dropdown when clicked', () => {
        const onCollapse = jest.fn();

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                onCollapse={ onCollapse }
                options={ getOptions() } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.click(node);

        expect(onCollapse).toHaveBeenCalled();
    });

    it('should collapse expanded dropdown when clicking outside', () => {
        const onCollapse = jest.fn();

        TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                onCollapse={ onCollapse }
                options={ getOptions() } />
        );

        document.dispatchEvent(new Event('mousedown'));
        expect(onCollapse).toHaveBeenCalled();
    });

    it('should collapse expanded dropdown when using escape', () => {
        const onCollapse = jest.fn();

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                onCollapse={ onCollapse } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 27 });

        expect(onCollapse).toHaveBeenCalled();
    });

    [
        { name: 'should collapse expanded dropdown when using enter', data: { which: 13 } },
        { name: 'should collapse expanded dropdown when using space', data: { which: 32 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onCollapse = jest.fn();

            const dropdown = TestUtils.renderIntoDocument(
                <TestDropdown
                    expanded={ true }
                    onCollapse={ onCollapse }
                    options={ getOptions() } />
            );

            const node = ReactDOM.findDOMNode(dropdown);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onCollapse).toHaveBeenCalled();
        });
    });

    it('should change value when clicking new option', () => {
        const onChange = jest.fn();

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                onChange={ onChange }
                options={ getOptions() }
                value="a" />
        );

        const options = TestUtils.scryRenderedDOMComponentsWithClass(dropdown, optionElement);
        expect(options.length).toBe(3);

        TestUtils.Simulate.click(ReactDOM.findDOMNode(options[1]));
        expect(onChange).toHaveBeenCalledWith('b');
    });

    [
        { name: 'should change value when using enter on new option', data: { which: 13 } },
        { name: 'should change value when using space on new option', data: { which: 32 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onChange = jest.fn();

            const dropdown = TestUtils.renderIntoDocument(
                <TestDropdown
                    expanded={ true }
                    highlightedIndex={ 1 }
                    onChange={ onChange }
                    options={ getOptions() }
                    value="a" />
            );

            const node = ReactDOM.findDOMNode(dropdown);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onChange).toHaveBeenCalledWith('b');
        });
    });

    [
        { name: 'should cancel when using enter on same option', data: { which: 13 } },
        { name: 'should cancel when using space on same option', data: { which: 32 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onChange = jest.fn();
            const onCollapse = jest.fn();

            const dropdown = TestUtils.renderIntoDocument(
                <TestDropdown
                    expanded={ true }
                    highlightedIndex={ 0 }
                    onChange={ onChange }
                    onCollapse={ onCollapse }
                    options={ getOptions() }
                    value="a" />
            );

            const node = ReactDOM.findDOMNode(dropdown);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onChange).not.toHaveBeenCalled();
            expect(onCollapse).toHaveBeenCalled();
        });
    });

    it('should be able to highlight option by typing its value', () => {
        const onHighlightedIndexChange = jest.fn();

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                onHighlightedIndexChange={ onHighlightedIndexChange }
                options={ getOptions() }
                value="a" />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyPress(node, { which: 66 }); // b

        expect(onHighlightedIndexChange).toHaveBeenCalledWith(1);
    });

    it('should change highlighted index when using down arrow when expanded', () => {
        const onHighlightedIndexChange = jest.fn();

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                highlightedIndex={ 0 }
                onHighlightedIndexChange={ onHighlightedIndexChange }
                options={ getOptions() } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 40 });

        expect(onHighlightedIndexChange).toHaveBeenCalledWith(1);
    });

    it('should change highlighted index when using up arrow when expanded', () => {
        const onHighlightedIndexChange = jest.fn();

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                highlightedIndex={ 2 }
                onHighlightedIndexChange={ onHighlightedIndexChange }
                options={ getOptions() } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 38 });

        expect(onHighlightedIndexChange).toHaveBeenCalledWith(1);
    });

    it('should skip disabled options when using down arrow when expanded', () => {
        const onHighlightedIndexChange = jest.fn();

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                highlightedIndex={ 0 }
                onHighlightedIndexChange={ onHighlightedIndexChange }
                options={ getOptions(false, true, false) } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 40 });

        expect(onHighlightedIndexChange).toHaveBeenCalledWith(2);
    });

    it('should skip disabled options when using up arrow when expanded', () => {
        const onHighlightedIndexChange = jest.fn();

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                highlightedIndex={ 2 }
                onHighlightedIndexChange={ onHighlightedIndexChange }
                options={ getOptions(false, true, false) } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 38 });

        expect(onHighlightedIndexChange).toHaveBeenCalledWith(0);
    });

    it('should wrap around highligthed index when using down arrow when expanded and on last option', () => {
        const onHighlightedIndexChange = jest.fn();

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                highlightedIndex={ 2 }
                onHighlightedIndexChange={ onHighlightedIndexChange }
                options={ getOptions() } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 40 });

        expect(onHighlightedIndexChange).toHaveBeenCalledWith(0);
    });

    it('should wrap around highlighted index when using up arrow when expanded and on first option ', () => {
        const onHighlightedIndexChange = jest.fn();

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                highlightedIndex={ 0 }
                onHighlightedIndexChange={ onHighlightedIndexChange }
                options={ getOptions() } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 38 });

        expect(onHighlightedIndexChange).toHaveBeenCalledWith(2);
    });

    [
        { name: 'should ignore up arrow key when all options are disabled', data: { which: 38 } },
        { name: 'should ignore down arrow key when all options are disabled', data: { which: 40 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onHighlightedIndexChange = jest.fn();

            const dropdown = TestUtils.renderIntoDocument(
                <TestDropdown
                    expanded={ true }
                    highlightedIndex={ 0 }
                    onHighlightedIndexChange={ onHighlightedIndexChange }
                    options={ getOptions(true, true, true) } />
            );

            const node = ReactDOM.findDOMNode(dropdown);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onHighlightedIndexChange).not.toHaveBeenCalled();
        });
    });

    it('should select first if no initial value when expanded and using down arrow ', () => {
        const onHighlightedIndexChange = jest.fn();

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                highlightedIndex={ -1 }
                onHighlightedIndexChange={ onHighlightedIndexChange }
                options={ getOptions() } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 40 });

        expect(onHighlightedIndexChange).toHaveBeenCalledWith(0);
    });

    it('should select last if no initial value when expanded and using up arrow ', () => {
        const onHighlightedIndexChange = jest.fn();

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                highlightedIndex={ -1 }
                onHighlightedIndexChange={ onHighlightedIndexChange }
                options={ getOptions() } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 38 });

        expect(onHighlightedIndexChange).toHaveBeenCalledWith(2);
    });

    [
        { name: 'should ignore up arrow key when all options are disabled', data: { which: 38 } },
        { name: 'should ignore down arrow key when all options are disabled', data: { which: 40 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onHighlightedIndexChange = jest.fn();

            const dropdown = TestUtils.renderIntoDocument(
                <TestDropdown
                    expanded={ true }
                    highlightedIndex={ -1 }
                    onHighlightedIndexChange={ onHighlightedIndexChange }
                    options={ [] } />
            );

            const node = ReactDOM.findDOMNode(dropdown);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onHighlightedIndexChange).not.toHaveBeenCalled();
        });
    });

});
