import React from 'react';
import ReactDOM from 'react-dom';
import TestUtils from 'react-addons-test-utils';
import purebem from 'purebem';

import RadioButtonGroup from './RadioButtonGroup';

import radioButtonOptionState from './redux/radioButtonOptionState';


const TestRadioButtonGroup = RadioButtonGroup({
    block: purebem.of('test-radio-button-group'),
    displayName: 'TestRadioButtonGroup'
});

const getOptions = (a = false, b = false, c = false) => {
    return [
        radioButtonOptionState({
            disabled: a,
            text: 'A',
            value: 'a'
        }),
        radioButtonOptionState({
            disabled: b,
            text: 'B',
            value: 'b'
        }),
        radioButtonOptionState({
            disabled: c,
            text: 'C',
            value: 'c'
        })
    ];
};

describe('RadioButtonGroup', () => {

    [
        { name: 'should change value when using left arrow', data: { which: 37 } },
        { name: 'should change value when using up arrow', data: { which: 38 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onChange = jest.fn();

            const group = TestUtils.renderIntoDocument(
                <TestRadioButtonGroup
                    onChange={ onChange }
                    options={ getOptions() }
                    value="b" />
            );

            const node = ReactDOM.findDOMNode(group);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onChange).toHaveBeenCalledWith('a');
        });
    });

    [
        { name: 'should change value when using right arrow', data: { which: 39 } },
        { name: 'should change value when using down arrow', data: { which: 40 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onChange = jest.fn();

            const group = TestUtils.renderIntoDocument(
                <TestRadioButtonGroup
                    onChange={ onChange }
                    options={ getOptions() }
                    value="b" />
            );

            const node = ReactDOM.findDOMNode(group);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onChange).toHaveBeenCalledWith('c');
        });
    });

    [
        { name: 'should skip disabled options using left arrow', data: { which: 37 } },
        { name: 'should skip disabled options using up arrow', data: { which: 38 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onChange = jest.fn();

            const group = TestUtils.renderIntoDocument(
                <TestRadioButtonGroup
                    onChange={ onChange }
                    options={ getOptions(false, true, false) }
                    value="a" />
            );

            const node = ReactDOM.findDOMNode(group);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onChange).toHaveBeenCalledWith('c');
        });
    });

    [
        { name: 'should skip disabled options using right arrow', data: { which: 39 } },
        { name: 'should skip disabled options using down arrow', data: { which: 40 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onChange = jest.fn();

            const group = TestUtils.renderIntoDocument(
                <TestRadioButtonGroup
                    onChange={ onChange }
                    options={ getOptions(false, true, false) }
                    value="c" />
            );

            const node = ReactDOM.findDOMNode(group);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onChange).toHaveBeenCalledWith('a');
        });
    });


    [
        { name: 'should ignore left arrow key when all options are disabled', data: { which: 37 } },
        { name: 'should ignore up arrow key when all options are disabled', data: { which: 38 } },
        { name: 'should ignore right arrow key when all options are disabled', data: { which: 39 } },
        { name: 'should ignore down arrow key when all options are disabled', data: { which: 40 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onChange = jest.fn();

            const group = TestUtils.renderIntoDocument(
                <TestRadioButtonGroup
                    onChange={ onChange }
                    options={ getOptions(true, true, true) }
                    value="a" />
            );

            const node = ReactDOM.findDOMNode(group);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onChange).not.toHaveBeenCalled();
        });
    });

    [
        { name: 'should wrap around when using left arrow', data: { which: 37 } },
        { name: 'should wrap around when using up arrow', data: { which: 38 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onChange = jest.fn();

            const group = TestUtils.renderIntoDocument(
                <TestRadioButtonGroup
                    onChange={ onChange }
                    options={ getOptions() }
                    value="a" />
            );

            const node = ReactDOM.findDOMNode(group);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onChange).toHaveBeenCalledWith('c');
        });
    });

    [
        { name: 'should wrap around when using right arrow', data: { which: 39 } },
        { name: 'should wrap around when using down arrow', data: { which: 40 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onChange = jest.fn();

            const group = TestUtils.renderIntoDocument(
                <TestRadioButtonGroup
                    onChange={ onChange }
                    options={ getOptions() }
                    value="c" />
            );

            const node = ReactDOM.findDOMNode(group);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onChange).toHaveBeenCalledWith('a');
        });
    });

    [
        { name: 'should select first if no initial value when using right arrow', data: { which: 39 } },
        { name: 'should select first if no initial value when using down arrow', data: { which: 40 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onChange = jest.fn();

            const group = TestUtils.renderIntoDocument(
                <TestRadioButtonGroup
                    onChange={ onChange }
                    options={ getOptions() }
                    value="" />
            );

            const node = ReactDOM.findDOMNode(group);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onChange).toHaveBeenCalledWith('a');
        });
    });

    [
        { name: 'should select last if no initial value when using left arrow', data: { which: 37 } },
        { name: 'should select last if no initial value when using up arrow', data: { which: 38 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onChange = jest.fn();

            const group = TestUtils.renderIntoDocument(
                <TestRadioButtonGroup
                    onChange={ onChange }
                    options={ getOptions() }
                    value="" />
            );

            const node = ReactDOM.findDOMNode(group);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onChange).toHaveBeenCalledWith('c');
        });
    });

    [
        { name: 'should ignore left arrow key when no options', data: { which: 37 } },
        { name: 'should ignore up arrow key when no options', data: { which: 38 } },
        { name: 'should ignore right arrow key when no options', data: { which: 39 } },
        { name: 'should ignore down arrow key when no option', data: { which: 40 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onChange = jest.fn();

            const group = TestUtils.renderIntoDocument(
                <TestRadioButtonGroup
                    onChange={ onChange }
                    options={ [] }
                    value="" />
            );

            const node = ReactDOM.findDOMNode(group);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onChange).not.toHaveBeenCalled();
        });
    });


});
